<?php
namespace Claromentis\Core\DAL;

$GLOBALS["__65da1b1ec2d6653e8a" . "0ab7848e536215"] = (@$GLOBALS["__65da1b1ec2d665" . "3e8a0ab7848e5362" . "15"] ?: array()) + array();




class EmptyResult implements ResultInterface
{
    public function fetchRow()
    {
        return null;
    }
    public function fetchArray()
    {
        return null;
    }
    
    
    
    
    
    
    public function numRows()
    {
        return 0;
    }
    public function hasData()
    {
        return false;
    }
    public function fetchAllValues()
    {
        return array();
    }
    public function free()
    {
    }
}